package com.opticon.samplesdk;

import com.extbcr.scannersdk.BarcodeManager;
import com.extbcr.scannersdk.EventListener;
import com.extbcr.scannersdk.CodeID;
import com.extbcr.scannersdk.BarcodeData;

import android.os.Bundle;
import android.app.Activity;
import android.graphics.Color;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.util.Log;

import android.os.Handler;

import android.graphics.BitmapFactory;
import android.widget.ImageView;
import android.graphics.Bitmap;

import android.os.Environment;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import android.Manifest;
import android.content.pm.PackageManager;
public class MainActivity extends Activity {
	private static final String TAG = "ScannerSDK-MainActivity";

	private EditText showScanResult;
	private TextView status;
	private Button mScan;
	private Button mTrigger;
	private int statusTextColor;
	private BarcodeManager mBarcodeManager;
	private EventListener mEventListener;

	private boolean ignoreStop = false;
	int i=0;
	private boolean triggermode = false;
	private boolean serverconnect = false;
	Handler mHandler;
	Runnable mRunnable;

	private Button mSnapshot;
	private Button mPreview;
	private ImageView mImageView;
	private boolean isPreviewMode = false;
	private boolean isPreviewStart = false;
	private Runnable mRunnablePreview;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.e(TAG, "onCreate: " + Thread.currentThread().getId());
		super.onCreate(savedInstanceState);
		Window window = getWindow();
		window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		setContentView(R.layout.activity_main);
		setupView();
		//initScan();
	}

	private void initScan() {
		Log.e(TAG, "initScan: XXX");
		mBarcodeManager = new BarcodeManager(this);
		mBarcodeManager.init();

		mEventListener = new EventListener() {

			@Override
			public void onReadData(BarcodeData result) {
				Log.e(TAG, "onReadData " + result.getText() + ", codeid is" + result.getCodeID());

				status.setTextColor(Color.GREEN);
				status.setText("Success");
				showScanResult.setText("");
				showScanResult.append("CodeID: " + CodeID.CodeIDtoString(result.getCodeID()) + "\n");
				String string = result.getText();
				if( string!= null) {
					if(string.length() > 200) {
						string = string.substring(0, 200);
						string = string + "[...]";
					}
					showScanResult.append("Result: " + string+ "\n");
				}
				ignoreStop = true;

			}

			@Override
			public void onTimeout() {
				Log.e(TAG, "onTimeout");

				status.setTextColor(Color.CYAN);
				status.setText("Timeout");
				ignoreStop = true;
			}

			@Override
			public void onConnect(){
				Log.e(TAG, "onConnect");
				serverconnect = true;

			}

			@Override
			public void onDisconnect(){
				Log.e(TAG, "onDisconnect");
				serverconnect = false;
			}

			@Override
			public void onStart(){
				Log.e(TAG, "onStart");
				status.setTextColor(Color.RED);
				status.setText("Scanning");
			}

			@Override
			public void onStop(){
				Log.e(TAG, "onStop");
				status.setTextColor(Color.WHITE);
				status.setText("Ready to Scan");
			}

			@Override
			public void onImgBuffer(byte[] imgdata, int type){
				Log.e(TAG, "onImgBuffer type=" + type + " imagesize=" + imgdata.length);

				if(isPreviewMode) {
					mHandler.postDelayed(mRunnablePreview, 1);
				}

				//Log.e(TAG, "onImgBuffer" + imgdata[0] + " ," + imgdata[1] + " ," + imgdata[2] + " ," + imgdata[3] + " ," + imgdata[4]);
				if(type == 3) {
					/*if(!isPreviewMode)
						createExternalStoragePublicPicture(imgdata, "bmp");*/
					Bitmap bmp = BitmapFactory.decodeByteArray(imgdata, 0, imgdata.length);
					mImageView.setImageBitmap(bmp);
				}

				if(type == 1) {
					if(!isPreviewMode) {
						createExternalStoragePublicPicture(imgdata, "jpg");
					}
					Bitmap bmp = BitmapFactory.decodeByteArray(imgdata, 0, imgdata.length);
					mImageView.setImageBitmap(bmp);
				}

			}
		};

		mBarcodeManager.addListener(mEventListener);
		mHandler = new Handler();
		mRunnablePreview = new Runnable(){
			@Override
			public void run() {
				if(serverconnect){
					mBarcodeManager.takeSnapshot(1, 8, 3, 65);
				}
			}
		};
	}

	private void setupView() {
		mImageView = (ImageView) findViewById(R.id.snapshot_view);
		showScanResult = (EditText) findViewById(R.id.scan_result);
		showScanResult.setKeyListener(null);

		status = (TextView) findViewById(R.id.wcccamera_title);
		statusTextColor = status.getCurrentTextColor();

		mScan = (Button) findViewById(R.id.scan);
		mScan.setOnTouchListener(new OnTouchListener() {

			@Override
			public boolean onTouch(View v, MotionEvent event) {
				if(event.getAction() == MotionEvent.ACTION_DOWN) {
					try {
						mScan.setPressed(true);
						if(serverconnect)
							mBarcodeManager.startDecode();
						mImageView.setImageBitmap(null);
						isPreviewMode = false;
						isPreviewStart = false;
						mPreview.setText("Preview");
						mHandler.removeCallbacks(mRunnablePreview);
					} catch (Exception e) {

					}
				} else if (event.getAction() == MotionEvent.ACTION_UP) {
					try {
						mScan.setPressed(false);
						if(serverconnect)
							mBarcodeManager.stopDecode();
					} catch (Exception e) {
					}
				}
				return true;
			}
		}
				);

		mTrigger = (Button) findViewById(R.id.trigger);
		//mTrigger.setVisibility(View.GONE);
		mTrigger.setOnTouchListener(new OnTouchListener() {

			@Override
			public boolean onTouch(View v, MotionEvent event) {
				if(event.getAction() == MotionEvent.ACTION_DOWN) {
					try {
						mTrigger.setPressed(true);
						if(triggermode){
							mTrigger.setText("TRIGGER");
							triggermode = false;
							if(serverconnect)
								mBarcodeManager.stopTrigger();
						}
						else {
							mTrigger.setText("STOP");
							showScanResult.setText("");
							triggermode = true;
							if(serverconnect)
								mBarcodeManager.startTrigger();
							mImageView.setImageBitmap(null);
							isPreviewMode = false;
							isPreviewStart = false;
							mPreview.setText("Preview");
							mHandler.removeCallbacks(mRunnablePreview);
						}
					} catch (Exception e) {

					}
				} else if (event.getAction() == MotionEvent.ACTION_UP) {
					try {
						mTrigger.setPressed(false);
					} catch (Exception e) {
					}
				}
				return true;
			}
		}
				);


		mSnapshot = (Button) findViewById(R.id.snapshot);
		mSnapshot.setOnTouchListener(new OnTouchListener() {

				@Override
				public boolean onTouch(View v, MotionEvent event) {
					if(event.getAction() == MotionEvent.ACTION_DOWN) {
						try {
							mSnapshot.setPressed(true);
						} catch (Exception e) {

						}
					} else if (event.getAction() == MotionEvent.ACTION_UP) {
						try {
							mSnapshot.setPressed(false);
							if(serverconnect) {
								isPreviewMode = false;
								isPreviewStart = false;
								mPreview.setText("Preview");
								mHandler.removeCallbacks(mRunnablePreview);
								//mBarcodeManager.takeSnapshot(1, 8, 3, 65);
								mBarcodeManager.takeSnapshot(1, 8, 1, 65);
							}
						} catch (Exception e) {
						}
					}
					return true;
				}
			}
		);

		mPreview = (Button) findViewById(R.id.preview);
		mPreview.setOnTouchListener(new OnTouchListener() {

				@Override
				public boolean onTouch(View v, MotionEvent event) {
					if(event.getAction() == MotionEvent.ACTION_DOWN) {
						try {
							mPreview.setPressed(true);
							if(isPreviewMode) {
								mPreview.setText("Preview");
								isPreviewMode = false;
								isPreviewStart = false;
								mHandler.removeCallbacks(mRunnablePreview);
							}
							else {
								mPreview.setText("Stop Preview");
								showScanResult.setText("");
								isPreviewMode = true;
								isPreviewStart = true;
								if(serverconnect) {
									mBarcodeManager.takeSnapshot(1, 8, 3, 65);
								}
							}

						} catch (Exception e) {

						}
					} else if (event.getAction() == MotionEvent.ACTION_UP) {
						try {
							mPreview.setPressed(false);
						} catch (Exception e) {
						}
					}
					return true;
				}
			}
		);
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	@Override
	protected void onPause() {
		super.onPause();
		try {
			if(serverconnect) mBarcodeManager.stopDecode();
			if(triggermode) { Log.e(TAG, "onPause triggermode is true, stopTrigger");  mBarcodeManager.stopTrigger(); }
			mBarcodeManager.removeListener();
			mBarcodeManager.deinit();
		} catch (Exception e) {
		}
	}

	protected void initParameter() {
		mTrigger.setText("TRIGGER");
		status.setText("Ready to Scan");
		status.setTextColor(Color.WHITE);
		triggermode = false;
		mImageView.setImageBitmap(null);
		mPreview.setText("Preview");
		isPreviewMode = false;
		isPreviewStart = false;
	}

	@Override
	protected void onResume() {
		super.onResume();
		showScanResult.setText(getResources().getText(R.string.scanner_hint));
		initParameter();
		initScan();
		requestPermission();
	}

	@Override
	protected void onStart() {
		super.onStart();
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		return super.onKeyDown(keyCode, event);
	}
	void createExternalStoragePublicPicture(byte[] imagedata, String extension_type) {

		File path = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES);
		String filetime = new SimpleDateFormat("_yyyyMMdd_HHmmss").format(new Date());
		File file = new File(path, "DemoPicture" + filetime + "." + extension_type);

		try {
			path.mkdirs();

			OutputStream os = new FileOutputStream(file);
			os.write(imagedata);
			os.close();

		} catch (IOException e) {
			Log.w(TAG, "Error writing " + file, e);
		}
	}

	private void requestPermission() {
		if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
			requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 0);
		} else {
			//Log.d(TAG, "requestPermission() permission has already been granted");
		}
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		switch (requestCode) {
			case 0: {
				// If request is cancelled, the result arrays are empty.
				if (grantResults.length > 0 &&
					grantResults[0] == PackageManager.PERMISSION_GRANTED) {
					//Log.d(TAG, "onRequestPermissionsResult() permission was granted");
				} else {
					finish();
				}
				return;
			}
		}
	}
}
